#!/bin/sh
# Version: 0.1.0
# Author: Dimitrios Dimitrakos mail@dimitrios.de
# $Id$

. ~/.vdrconvert/vdrconvert.env

# Default Directorys
[ -z "$VDRROOT" ]           && export VDRROOT=/video
[ -z "$DESTDIR" ]           && export DESTDIR=/video
[ -z "$TEMPDIR" ]           && export TEMPDIR=$DESTDIR/tmp
[ -z "$LOGDIR" ]            && export LOGDIR=/var/log/vdrconvert
[ -z "$VCORUNDIR" ]         && export VCORUNDIR=/var/run/vdrconvert
[ -z "$VCOQUEUEDIR" ]       && export VCOQUEUEDIR=/var/spool/vdrconvert

PIDFILE="$VCORUNDIR/`basename ${0}.pid`"
echo "$$" >> $PIDFILE

trap do_clean SIGINT SIGQUIT SIGTERM 
do_clean() {
	echo "$0 aborted by signal"
	logger "$0 aborted by signal"
	if [ -f $PIDFILE ]
	then
	    TEMPPID=/tmp/`basename $PIDFILE`.$$
	    mv $PIDFILE $TEMPPID
	    status=$?
	    if [ $status -eq 0 ]
	    then
	        [ `cat $TEMPPID|wc -l` -gt 0 ] && cat $TEMPPID|grep -v $$ > $PIDFILE 
	    fi
	    [ -f $TEMPPID ] && rm $TEMPPID
	fi
	exit
}


[ "$DEBUG" = "yes" ] && set -x 
echo "`date` : Start $0"

# Default values for executables
[ -z "$VDRSYNC" ]           && export VDRSYNC=$VDRCONVERTBINDIR/vdrsync.pl
[ -z "$TCMPLEX" ]           && export TCMPLEX=tcmplex
[ -z "$DVBMPLEX" ]          && export DVBMPLEX=dvb-mplex
[ -z "$MPLEX" ]             && export MPLEX=mplex
[ -z "$TCMPLEX_PANTELTJE" ] && export TCMPLEX_PANTELTJE=tcmplex-panteltje
[ -z "$MKISOFS" ]           && export MKISOFS=mkisofs
[ -z "$MENCODER" ]          && export MENCODER=mencoder
[ -z "$LAME" ]              && export LAME=lame
[ -z "$DVDAUTHOR" ]         && export DVDAUTHOR=dvdauthor
[ -z "$TOSVCD" ]            && export TOSVCD=tosvcd
[ -z "$COMPRESS" ]          && export COMPRESS="bzip2 -f"
# Nice level
[ -z "$PRIO" ]              && export PRIO=15
[ -z "$SLEEP_TIME" ]        && export SLEEP_TIME=10
[ -z "$CHECK_SLEEP_TIME" ]  && export CHECK_SLEEP_TIME=5
# Schalter
[ -z "$PARQ" ]              && export PARQ="none"
[ -z "$CONV_TITLEDATE" ]    && export CONV_TITLEDATE=no
# Project X
[ -z "$LS" ]                && export LS="ls"
[ -z "$XVFB" ]              && export XVFB=Xvfb
# Java Pfad
[ -z "$JAVA" ]              && export JAVA=java
[ -z "$PX" ]                && export PX=$VDRCONVERTDIR/share/vdrconvert/pX/pX.jar
[ -z "$PXINI" ]             && export PXINI=$VDRCONVERTDIR/share/vdrconvert/pX/pX.ini
[ -z "$KILL" ]              && export KILL=killall
# Wine pfad
[ -z "$WINE" ]              && export WINE=wine
# Jobs
[ -z "$CONVERT_JOBS" ]      && export CONVERT_JOBS="vdr2divx vdr2dvd vdr2vcd vdr2svcd vdr2mpg vdr2mp3 vdr2ac3 burn"
# Monitor
[ -z "$OSDMONITOR" ]        && export OSDMONITOR="yes"

if [ ! -d "$VDRROOT" ]
then
	echo "ERROR : VDRROOT is not available."
	logger "$0 : ERROR : VDRROOT is not available."
	exit 1
fi

case $DEMUXER in
	pva|px)
	      echo "Start Virtual Server"
              $XVFB :1 >/dev/null 2>&1 &
	      export DISPLAY=:1.0
	      ;;
	*)
	      ;;
esac

while true; do	

 [ -d $VCORUNDIR ]      || mkdir -p $VCORUNDIR
 [ -d $TEMPDIR ] 	|| mkdir -p $TEMPDIR
 [ -d $LOGDIR ] 	|| mkdir -p $LOGDIR
 [ -d $VCOQUEUEDIR ] 	|| mkdir -p $VCOQUEUEDIR
 [ -d $TEMPDIR ] 	|| mkdir -p $TEMPDIR
 [ -d $DESTDIR ] 	|| mkdir -p $DESTDIR
 
 if [ -w ${TEMPDIR} -a -w ${LOGDIR} -a -w $VCOQUEUEDIR -a -w $TEMPDIR -a -w $DESTDIR -a -w $VCORUNDIR ]
 then
  for JOBFIL in $CONVERT_JOBS
  do
    if [ ! -f $VCOQUEUEDIR/$JOBFIL ]
    then 
	touch $VCOQUEUEDIR/$JOBFIL
    fi

    let LINES=0
    LINES=`cat $VCOQUEUEDIR/$JOBFIL | wc -l`  
    
    if [ $LINES -gt 0 ]
    then
      let run=0
      if [ "$PARQ" = "no" ]
      then
	run=`ps -aef|grep $VDRCONVERTBINDIR/${JOBFIL}.sh|grep -v grep|wc -l`
	#JOBPIDFILE="$VCORUNDIR/`basename ${JOBFIL}.sh.pid`"
	#[ -f $JOBPIDFILE ] && let run=`cat $JOBPIDFILE|wc -l`
      fi
      if [ "$PARQ" = "none" ]
      then
	ps -aef > $TEMPDIR/vcorun.$$
        for r in $CONVERT_JOBS
	do
	    if [ $run -eq 0 ]
	    then
		#run=`ps -aef|grep ${r}.sh|grep -v grep|wc -l`
		run=`cat $TEMPDIR/vcorun.$$|grep ${r}.sh|grep -v grep|wc -l`
		#JOBPIDFILE="$VCORUNDIR/`basename ${r}.sh.pid`"
		#[ -f $JOBPIDFILE ] && let run=`cat $JOBPIDFILE|wc -l`

	    fi
	done
	[ -f $TEMPDIR/vcorun.$$ ] && rm $TEMPDIR/vcorun.$$
      fi

      if [ $run -eq 0 ]
      then
          NewJobs=`mktemp ${TEMPDIR}/${JOBFIL}.XXXXXX`
	  status=$?
	  if [ $status -eq 0 ]
	  then
    	      # Define Logfile
    	      CONVERTLOG=${LOGDIR}/${JOBFIL}.log

	      echo "Start $VDRCONVERTBINDIR/${JOBFIL}.sh $NewJobs $JOBFIL Log = $CONVERTLOG" 
	      $VDRCONVERTBINDIR/${JOBFIL}.sh $NewJobs $JOBFIL $CONVERTLOG >> $CONVERTLOG 2>&1 &
      	      logger "${JOBFIL}.sh $NewJobs $JOBFIL  $CONVERTLOG 2>&1 gestartet"
	  else
	      echo "ERROR : mktemp ${TEMPDIR}/${JOBFIL}.XXXXXX failed."
	      logger " $0 : ERROR : mktemp ${TEMPDIR}/${JOBFIL}.XXXXXX failed." 
	  fi
      fi
    fi
  done
  sleep $CHECK_SLEEP_TIME
 else
      echo "ERROR : Wrong permission . $0 exit"
      logger "$0 : ERROR : Wrong permissions. $0 exit"
 fi
done

# XXX Nur von uns gestartetes Display killen
case $DEMUXER in
	pva|px)
              $KILL -15 $XVFB >/dev/null 2>&1
	      sleep 1
              $KILL -9 $XVFB >/dev/null 2>&1
	      ;;
	*)
	      ;;
esac
